@@ -26,7 +26,6 @@
 #include <http_parser.h>
 #include <ccan/htable/htable.h>
 #include <ccan/list/list.h>
-#include <syslog.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <unistd.h>
@@ -49,14 +48,8 @@
 #endif
 
 #define MAX_MSG_SIZE 16*1024
+#define DTLS_PROTO_INDICATOR "PSK-NEGOTIATE"
 
-enum {
-	PS_AUTH_INACTIVE, /* no comm with worker */
-	PS_AUTH_FAILED, /* no tried authenticated but failed */
-	PS_AUTH_INIT, /* worker has sent an auth init msg */
-	PS_AUTH_CONT, /* worker has sent an auth cont msg */
-	PS_AUTH_COMPLETED /* successful authentication */
-};
 
 typedef enum {
 	SOCK_TYPE_TCP,
@@ -119,16 +112,6 @@ inline static const char *proto_to_str(fw_proto_t proto)
 /* The time after a disconnection the cookie is valid */
 #define DEFAULT_COOKIE_RECON_TIMEOUT 120
 
-/* Timeout (secs) for communication between main and sec-mod */
-#define MAIN_SEC_MOD_TIMEOUT 120
-#define MAX_WAIT_SECS 3
-
-#define DEBUG_BASIC 1
-#define DEBUG_INFO  3
-#define DEBUG_HTTP  4
-#define DEBUG_TRANSFERRED 5
-#define DEBUG_SENSITIVE 8
-#define DEBUG_TLS   9
 
 #define DEFAULT_DPD_TIME 600
 
@@ -159,34 +142,7 @@ extern int syslog_open;
 #define ACCT_TYPE_PAM (1<<1)
 #define ACCT_TYPE_RADIUS (1<<2)
 
-/* User Disconnect reasons (must be > 0) */
-#define REASON_ANY 1
-#define REASON_USER_DISCONNECT 2
-#define REASON_SERVER_DISCONNECT 3
-#define REASON_IDLE_TIMEOUT 4
-#define REASON_DPD_TIMEOUT 5
-#define REASON_ERROR 6
-#define REASON_SESSION_TIMEOUT 7
-
-#define ERR_SUCCESS 0
-#define ERR_BAD_COMMAND -2
-#define ERR_AUTH_FAIL -3
-#define ERR_AUTH_CONTINUE -4
-#define ERR_WAIT_FOR_SCRIPT -5
-#define ERR_MEM -6
-#define ERR_READ_CONFIG -7
-#define ERR_NO_IP -8
-#define ERR_PARSING -9
-#define ERR_EXEC -10
-#define ERR_PEER_TERMINATED -11
-#define ERR_CTL -12
-#define ERR_NO_CMD_FD -13
-
-#define ERR_WORKER_TERMINATED ERR_PEER_TERMINATED
-
-#define LOG_HTTP_DEBUG 2048
-#define LOG_TRANSFER_DEBUG 2049
-#define LOG_SENSITIVE 2050
+#include "defs.h"
 
 /* Allow few seconds prior to cleaning up entries, to avoid any race
  * conditions when session control is enabled.
@@ -197,41 +153,6 @@ extern int syslog_open;
 #define MAX_CIPHERSUITE_NAME 64
 #define SID_SIZE 32
 
-typedef enum {
-	AUTH_COOKIE_REP = 2,
-	AUTH_COOKIE_REQ = 4,
-	RESUME_STORE_REQ = 6,
-	RESUME_DELETE_REQ = 7,
-	RESUME_FETCH_REQ = 8,
-	RESUME_FETCH_REP = 9,
-	CMD_UDP_FD = 10,
-	CMD_TUN_MTU = 11,
-	CMD_TERMINATE = 12,
-	CMD_SESSION_INFO = 13,
-	CMD_BAN_IP = 16,
-	CMD_BAN_IP_REPLY = 17,
-
-	/* from worker to sec-mod */
-	CMD_SEC_AUTH_INIT = 120,
-	CMD_SEC_AUTH_CONT,
-	CMD_SEC_AUTH_REPLY,
-	CMD_SEC_DECRYPT,
-	CMD_SEC_SIGN,
-	CMD_SEC_CLI_STATS,
-
-	/* from main to sec-mod and vice versa */
-	MIN_SECM_CMD=239,
-	CMD_SECM_SESSION_OPEN, /* sync: reply is CMD_SECM_SESSION_REPLY */
-	CMD_SECM_SESSION_CLOSE, /* sync: reply is CMD_SECM_CLI_STATS */
-	CMD_SECM_SESSION_REPLY,
-	CMD_SECM_BAN_IP,
-	CMD_SECM_BAN_IP_REPLY,
-	CMD_SECM_CLI_STATS,
-	CMD_SECM_LIST_COOKIES,
-	CMD_SECM_LIST_COOKIES_REPLY,
-
-	MAX_SECM_CMD,
-} cmd_request_t;
 
 struct vpn_st {
 	char name[IFNAMSIZ];
@@ -346,6 +267,12 @@ struct cfg_st {
 	unsigned ban_points_connect;
 	unsigned ban_points_kkdcp;
 
+	/* when using the new PSK DTLS negotiation make sure that
+	 * the negotiated DTLS cipher/mac matches the TLS cipher/mac. */
+	unsigned match_dtls_and_tls;
+	unsigned dtls_psk; /* whether to enable DTLS-PSK */
+	unsigned dtls_legacy; /* whether to enable DTLS-LEGACY */
+
 	unsigned isolate; /* whether seccomp should be enabled or not */
 
 	unsigned auth_timeout; /* timeout of HTTP auth */
